import moment from 'moment'
import { log } from './LogController.js'
import InvoiceRepository from '../Repositories/InvoiceRepository.js'
import { doPaymentThroughAPI, confirmPayment, cancelPayment, consultPayment } from './PaymentController.js'
import EmailController from './EmailController.js'

const getInvoices = async function ({ NRCPFRESPCON, CDCONSUMIDOR }) {
    let invoices = []
    
    try {
        invoices = await InvoiceRepository.getInvoices({ CPFRESPONSAVEL: 'F' + NRCPFRESPCON, CDCONSUMIDOR })
        if (invoices?.length > 0) invoices = formatDataToSend(invoices)
    } catch (error) {
        invoices = []
        log({ type: 'error', message: `Ocorreu um erro em getInvoices:\n${error.message}` })
    }
    
    return invoices
}

const formatDataToSend = (invoices) => {
    const formattedInvoices = []
    
    for (let i = 0; i < invoices.length; i++) {
        const invoice = invoices[i]
        let invoiceDueDate = moment(invoice.DTVENBOLREC).locale('pt-br')
        const monthIndex = formattedInvoices.findIndex(element => element.maturationDate === invoiceDueDate.format('DD/MMM'))
        const invoiceMonthNotExistsInArray = monthIndex === -1

        if (i === 0 || invoiceMonthNotExistsInArray) {
            let currentInvoice = {
                month: invoiceDueDate.format('MMM'),
                year: invoiceDueDate.format('YYYY'),
                maturationDate: invoiceDueDate.format('DD/MMM'),
                amount: invoice.VRFINALBREC,
                invoiceDetails: [ invoice ]
            }

            formattedInvoices.push(currentInvoice)
        } else {
            formattedInvoices[monthIndex].amount = Math.round((parseFloat(formattedInvoices[monthIndex].amount) + parseFloat(invoice.VRFINALBREC)) * 100) / 100
            formattedInvoices[monthIndex].invoiceDetails.push(invoice)
        }
    }

    for (let i = 0; i < formattedInvoices.length; i++) {
        const month = formattedInvoices[i]
        let monthInvoices = month.invoiceDetails
        let hasAnyUnpaidInvoice = false
        
        for (let j = 0; j < monthInvoices.length; j++) {
            if (monthInvoices[j].IDSTATUS !== 'B') hasAnyUnpaidInvoice = true
        }
        
        // 1 = pago, 0 = em aberto
        month.status = hasAnyUnpaidInvoice ? 0 : 1
    }
    
    return formattedInvoices
}

const doCardInvoicePayment = async ({ totalValue, flag, cardExpiration, cardNumber, cardToken, cardCVV, cardHolder, consDetails, mode, finalNumbers, idBoletos }) => {
    const { NMRESPCONS, DSEMAILCONS, CDFILIAL } = consDetails
    let nsu = ''
    let nit = ''
    let NRTRANSACAO = CDFILIAL + idBoletos[0]

    try {
        let response = null

        if (cardNumber) {
            response = await doPaymentThroughAPI({ totalValue, flag, cardExpiration, cardNumber, cardCVV, cardHolder, mode, NRTRANSACAO })
        } else {
            response = await doPaymentThroughAPI({ totalValue, flag, cardExpiration, cardToken, cardCVV, cardHolder, mode, NRTRANSACAO })
        }
    
        if (!response || response.status === 500) {
            return {
                error: true,
                message: 'Erro ao realizar pagamento.'
            }
        }
        
        nsu = response?.data?.result?.nsu || ''
        nit = response?.data?.result?.nit || ''

        console.log('NRTRANSACAO', NRTRANSACAO)
        console.log('nsu', nsu)
        console.log('nit', nit)        

        const confirmationResponse = await confirmPayment(nit)

        if (!confirmationResponse || confirmationResponse.status === 500) {
            return {
                error: true,
                message: 'Erro ao realizar pagamento.'
            }
        } else if (idBoletos?.length > 0) {
            // Atualiza a tabela do financeiro
            for (let i = 0; i < idBoletos.length; i++) {
                await InvoiceRepository.updateCreditCardIncome({ NSU: nsu, NRLANBOLREC: idBoletos[i], CDFILIAL })
            }
        }

        // send confirmation email
        let cardCode = 'XXXX XXXX XXXX '
        cardCode += finalNumbers ? finalNumbers : cardNumber.substring(12)
        const transactionValue = parseFloat(totalValue).toLocaleString('pt-br', { style: 'currency', currency: 'BRL' })
        const transactionDate = moment().format('DD/MM/YYYY HH:mm:ss')
        let docsNumbers = idBoletos.join(', ')

        const emailData = { name: NMRESPCONS, cardCode, transactionValue, transactionDate, docsNumbers }

        const emailResult = await EmailController.sendEmail({
            DSEMAILCONS,
            subject: 'Comprovante de Pagamento de Título',
            consDetails,
            htmlFunction: 'getCardInvoicePaymentConfirmationEmail',
            functionParams: emailData
        })

        if (emailResult?.accepted?.length > 0) {
            return {
                error: false,
                message: 'Pagamento realizado com sucesso. Comprovante enviado por e-mail.'
            }
        } else {
            return {
                error: false,
                message: 'Pagamento realizado com sucesso. Não foi possível enviar o comprovante por e-mail.'
            }
        }
    } catch (error) {
        log({ type: 'error', message: `Ocorreu um erro em doCardInvoicePayment\nNSU: ${nsu}\nNRTRANSACAO: ${NRTRANSACAO}\nNIT: ${nit}\n${error.message}` })

        await cancelPayment(nit)

        return {
            error: true,
            message: 'Erro ao realizar pagamento.'
        }
    }
    // finally {
    //     // O finally não é necessário na rotina, só está aqui para desenvolvimento e homologação
    //     // Não manter em produção
    //     let paymentResult = await consultPayment(nit)
    //     console.log(paymentResult.data)
    // }
}

export { getInvoices, doCardInvoicePayment }
